#!/usr/bin/perl

package Perceus::Link;

use Perceus::Util;
use Perceus::Socket;
use Exporter;
use strict;

our @ISA = qw(Exporter);

our @EXPORT = qw (

);

my $MAGIC = "PRCS20";
my $PROTOID = "    1";

=head1 NAME

Perceus::Link - Perceus packet interface

=head1 ABOUT


=head1 SYNOPSIS

    use Perceus::Packet;

=head1 METHODS

=over 12
=cut


=item connect()

=cut
sub
connect($$)
{
    my $proto               = shift;
    my $server              = shift || "127.0.0.1";
    my $class               = ref($proto) || $proto;
    my $self                = ();
    my $sock                = ();

    %{$self} = ();

    bless($self, $class);


    if ( $server eq "127.0.0.1" ) {
        $sock = IO::Socket::UNIX->new(Peer => $Perceus::Include::perceusd_socket);

        print "connected to local unix socket\n";

    } elsif ( defined($server) ) {
        $sock = IO::Socket::INET->new(PeerAddr => $server,
                                      PeerPort => '9870',
                                      Proto    => 'tcp');

        print "connected to $server\n";
    }


    $self->{"SOCK"} = $sock;
    $self->{"PACKET"} = Perceus::Packet->new();

    my $packet = $self->recieve();
    my $p = Perceus::Packet->new(\$packet);

    if ( $p->command() eq "HELLO" ) {
        &pprint(1, "DEBUG: Connected to Perceus server\n");
        $p->data("Hello from Perceus client $Perceus::Include::ID");
        $self->send($p->serialize());
    } else {
        die "Could not connect to Perceus server: ". $p->command() ."\n";
    }

    return $self;
}

=item recieve_until()


=cut
sub
recieve_until($$)
{



}



=item listen()

=cut
sub
listen($)
{
    my $proto               = shift;
    my $class               = ref($proto) || $proto;
    my $self                = ();

    %{$self} = ();

    bless($self, $class);


    &pprint("2", "Creating the select object\n");
    # Create the select object
    $self->{"SELECT"} = new IO::Select();

    &pprint("2", "Creating the socket object\n");
    # Create the listening socket and add it to the select object
    $self->{"SOCK"} = IO::Socket::INET->new( Proto     => 'tcp',
                                             LocalPort => 9870,
                                             Listen    => 1024,
                                             Reuse     => 1);

    die "Could not create socket: $!\n" unless $s;

    return($self);
}



















=item get_header()

Parse a raw string to fill the packet header.

=cut
sub
get_header($$)
{
    my $self                = shift;
    my $string              = shift;
    my @fields              = ();
    my $packet              = ();

    if ( ref($string) ) {
        $packet = $$string;
    } else {
        $packet = $string;
    }

    if ( length($string) < ( 6 + 8 + 8 + 16 + 8 ) ) {
        &pprint(1, "ERROR: Packet header too small\n");
        return();
    }

    if ( $packet =~ /^$MAGIC(.{8})(.{8})(.{16})(.{8})/ ) {
        $self->{"XID"} = &trim($1);
        $self->{"CID"} = &trim($2);
        $self->{"COMMAND"} = &trim($3);
        $self->{"LENGTH"} =  &trim($4);
    }
    return(1);
}


=item req_length()

How many bytes are we missing for a full packet

=cut
sub
req_length($$)
{
    my $self                = shift;

    if ( exists($self->{"COMMAND"}) and exists($self->{"LENGTH"}) ) {
        return($self->{"LENGTH"} - length($self->{"DATA"}) )
    }

    # If we don't have a good packet header, then we return the packet header
    # size
    return(6 + 8 + 8 + 16 + 8);
}


=item get_data()

Parses a raw socket string looking for properly formatted PRCS20 packet
header. Anything not formated correctly or left over will be returned back.

=cut
sub
get_data($$)
{
    my $self                = shift;
    my $string              = shift;
    my @fields              = ();
    my $packet              = ();
    my $length              = ();

    if ( ref($string) ) {
        $packet = $$string;
    } else {
        $packet = $string;
    }

    $length = $self->{"LENGTH"} - length($self->{"DATA"});

    if ( $length < $string ) {
        &pprint(1, "ERROR: Packet data too small\n");
        return();
    }

    if ( $length <= 0 ) {
        &pprint(1, "ERROR: I have all the data I need\n");
        return();
    }

    $self->{"DATA"} .= substr($string, 0, $length);


    return(1);
}


=item fill()

Fills up a packet structure via a raw string. It will return with anything
leftover in the passed string that doesn't fill up the packet.

=cut
sub
fill($$)
{
    my $self                    = shift;
    my $string                  = shift;
    my $packet                  = ();

    if ( ref($string) ) {
        $packet = $$string;
    } else {
        $packet = $string;
    }

print "fill: $packet\n";

    if ( $packet =~ /^$MAGIC/ ) {
        my $len = 6 - length($self->{"MAGIC"});
        if ( $packet =~ s/^(.{0,$len})// ) {
            $self->{"MAGIC"} .= $1;
            &pprint(2, "Adding MAGIC to packet object\n");
            &pprint(3, "   ->($len): $1\n");
        }
    }

    if ( $self->{"MAGIC"} eq $MAGIC ) {

        if ( ( ! exists($self->{"XID"}) or length($self->{"XID"}) < 8 ) and length($packet) > 0 ) {
            my $len = 8 - length($self->{"XID"});
            if ( $packet =~ s/^(.{0,$len})// ) {
                $self->{"XID"} .= $1;
                &pprint(2, "Adding XID to packet object\n");
                &pprint(3, "   ->($len): $1\n");
            }
        }

        if ( ( ! exists($self->{"CID"}) or length($self->{"CID"}) < 8 ) and length($packet) > 0 ) {
            my $len = 8 - length($self->{"CID"});
            if ( $packet =~ s/^(.{0,$len})// ) {
                $self->{"CID"} .= $1;
                &pprint(2, "Adding CID to packet object\n");
                &pprint(3, "   ->($len): $1\n");
            }
        }

        if ( ( ! exists($self->{"COMMAND"}) or length($self->{"COMMAND"}) < 8 ) and length($packet) > 0 ) {
            my $len = 16 - length($self->{"COMMAND"});
            if ( $packet =~ s/^(.{0,$len})// ) {
                $self->{"COMMAND"} .= $1;
                &pprint(2, "Adding COMMAND to packet object\n");
                &pprint(3, "   ->($len): $1\n");
            }
        }

        if ( ( ! exists($self->{"LENGTH"}) or length($self->{"LENGTH"}) < 8 ) and length($packet) > 0 ) {
            my $len = 8 - length($self->{"LENGTH"});
            if ( $packet =~ s/^(.{0,$len})// ) {
                $self->{"LENGTH"} .= $1;
                &pprint(2, "Adding LENGTH to packet object\n");
                &pprint(3, "   ->($len): $1\n");
            }
        }

        if ( ( ! exists($self->{"DATA"}) or length($self->{"DATA"}) < $self->{"LENGTH"} ) and length($packet) > 0 ) {
            my $len = $self->{"LENGTH"} - length($self->{"DATA"});
            if ( $packet =~ s/^(.{0,$len})// ) {
                $self->{"DATA"} .= $1;
                &pprint(2, "Adding DATA to packet object\n");
                &pprint(3, "   ->($len): $1\n");
            }
        }
    } else {
        if ( $packet =~ /^.*($MAGIC.*)$/ ) {
            $packet = $1; # Trim up to the MAGIC
        } else {
            $packet = (); # No MAGIC in string
        }
    }

    return($packet);
}


=item parse()

Parses a raw socket string looking for properly formatted packets. Anything
not formated correctly or left over will be returned back.

=cut
sub
parse($$)
{
    my $self                = shift;
    my $string              = shift;
    my @fields;
    my $packet              = ();

    if ( ref($string) ) {
        $packet = $$string;
    } else {
        $packet = $string;
    }
    if ( $packet =~ /^$MAGIC(.{8})(.{8})(.{16})(.{8})/ ) {
        $self->{"XID"} = &trim($1);
        $self->{"CID"} = &trim($2);
        $self->{"COMMAND"} = &trim($3);
        $self->{"LENGTH"} =  &trim($4);
        if ( $self->{"LENGTH"} > 0 ) {
            my $len = $self->{"LENGTH"};
            if ( $packet =~ /^$MAGIC.{40}(.{$len})/ ) {
                $self->{"DATA"} = $1;
                if ( $packet =~ s/^($MAGIC.{40}.{$len})// ) {
                    &pprint(3, "Found good packet: $self->{COMMAND}: $self->{DATA}\n");
                    &pprint(4, "removed old info: $1\n");
                }
            } else {
                #print "Did not get proper length of data\n";
            }
        }
    }
    return($packet);
}

=item xid()

Will set or print the XID region of the packet. Optional argument will set.
Maximum size is 8 bytes.

=cut
sub
xid($$)
{
    my $self                = shift;
    my $data                = shift;
    my $return              = ();

    if ( $data ) {

        if ( length($data) <= 8 ) {
            $return = 1;
            $self->{"XID"} = $data;
        }

    } else {
        $return = $self->{"XID"};
    }

    return($return);
}

=item uid()

Will set or print the CID region of the packet. Optional argument will set.
Maximum size is 8 bytes.

=cut
sub
uid($$)
{
    my $self                = shift;
    my $data                = shift;
    my $return              = ();

    if ( $data ) {

        if ( length($data) <= 8 ) {
            $return = 1;
            $self->{"CID"} = $data;
        }

    } else {
        $return = $self->{"CID"};
    }

    return($return);
}

=item command()

Will set or print the COMMAND region of the packet. Optional argument will set.
Maximum size is 16 bytes.

=cut
sub
command($$)
{
    my $self                = shift;
    my $data                = shift;
    my $return              = ();

    if ( $data ) {

        if ( length($data) <= 16 ) {
            $return = 1;
            $self->{"COMMAND"} = $data;
        }

    } else {
        $return = $self->{"COMMAND"};
    }

    return($return);
}

=item data()

Will set or print the DATA region of the packet. Optional argument will set.
Maximum size is 1024 bytes.

=cut
sub
data($$)
{
    my $self                = shift;
    my $data                = shift;
    my $return              = ();

    if ( $data ) {

        if ( length($data) <= 1024 ) {
            $return = 1;
            $self->{"DATA"} = $data;
            $self->{"LENGTH"} = length($data);
        }

    } else {
        $return = $self->{"DATA"};
    }

    return($return);
}


=item satisified()

If the packet is properly populated, this will return true.

=cut
sub
satisified($)
{
    my $self                = shift;

    return((exists($self->{"DATA"}) and exists($self->{"LENGTH"}) and $self->{"LENGTH"} == length($self->{"DATA"}) ? 1 : undef) );
}

=item clear()

Clear the object contents.

=cut
sub
clear($)
{
    my $self                = shift;

    %{$self} = ();
}

=item pack()

Pack the object into a string suitable for sending over a socket

=cut
sub
pack($)
{
    my $self                = shift;

#    printf("packed: %6.6s%8.8s%8.8s%16.16s%8.8s%s\n",
#        $MAGIC,
#        $self->{"XID"} || "",
#        $self->{"CID"} || "",
#        $self->{"COMMAND"} || "",
#        $self->{"LENGTH"} || length($self->{"DATA"}),
#        $self->{"DATA"} || ""
#    );
    return(sprintf("%-6.6s%-8.8s%-8.8s%-16.16s%-8.8s%s",
        $MAGIC,
        $self->{"XID"} || "",
        $self->{"CID"} || "",
        $self->{"COMMAND"} || "",
        $self->{"LENGTH"} || length($self->{"DATA"}),
        $self->{"DATA"} || ""
    ));
}

=back

=head1 SEE ALSO

Perceus::Network, Perceus::Link

=head1 COPYRIGHT

Perceus is copyright Arthur Stevens and Infiscale, Inc.

=cut

1;
