#!/usr/bin/perl

package Perceus::Network;

use IO::Socket;
use Perceus::Util;
use Exporter;

our @ISA = ('Exporter');

our @EXPORT = qw(
    &gen_uuid
    &new
    &send
    &recieve
    &recieve_until
    &recieve_packet
);


sub
gen_uuid($)
{

   my $size             = shift || "7";
   my @alphanumeric     = ('a'..'z');
   my $randstring       = join '', map $alphanumeric[rand @alphanumeric], 0..$size;

   return $randstring;
}


sub
send($$$$)
{
    my $sock = shift;
    my $type = shift;
    my $data = shift;
    my $uuid = shift;
    my $fileno = fileno($sock);
    my $return = ();

    if ( defined($type) ) {
        my $len = length($data);

        if ( ! $uuid ) {
            $uuid = "";
            #$uuid = &gen_uuid();
        }

        if ( $sock->syswrite(sprintf("%-16s%-8s%-8s%s", $type, $uuid, $len, $data)) ) {
            &pprint("2", "Sent     %8s: %-16s %-8s %-8s %s\n", "($fileno)", $type, $uuid, $len, $data);
            $return = $uuid;
        } else {
            &pprint(1, "ERROR      %8s:%-16s %-8s %-8s %s\n", "($fileno)", $type, $uuid, $len, $data);
        }
    } else {
        &pprint(1, "ERROR: Called send without all fields ($type, $data)\n");
    }

    return($return);
}


sub
recieve_data($$)
{
    my $sock = shift;
    my $length = shift;
    my ($buff, $size);

    if ( $sock->opened() ) {

        if ( defined($length) ) {
            my $read = 0;
            my $readlen = 0;
            do {
                my $tmpbuff;
                $readlen = $sock->sysread($tmpbuff, $length - $read) || 0;
                $read += $readlen;
                $buff .= $tmpbuff;
                &pprint(4, "Read: $read of $length: '$buff'\n");
            } while ( $read > 0 and $readlen > 0 and $read < $length );
            if ( $read != $length ) {
                &pprint(1, "Recieved bad data length ($length): $buff\n");
            }
# FIX: This shouldn't be done here...
            $buff =~ s/\s*$//;
            &pprint("3", "Read: $buff\n");
        } else {
            &pprint(1, "ERROR: Called recieve without data length!\n");
        }
    } else {
        &pprint(1, "ERROR: Is socket opened?\n");
    }

    return($buff);
}

sub
recieve_packet($)
{
    my $sock = shift;
    my $fileno = fileno($sock);
    my $type = "";
    my $uuid = "";
    my $length = "";
    my $data = "";

    $type = recieve_data($sock, 16);
    if ( $type ) {
        $uuid = recieve_data($sock, 8);
        $length = recieve_data($sock, 8);
        $data = recieve_data($sock, $length);
    }

    &pprint("2", "Recieved %8s: %-16s %-8s %-8s %s\n", "($fileno)", $type, $uuid, $length, $data);

    return($type, $uuid, $data);
}

sub
recieve_until($)
{
    my $sock = shift;
    my $readto = shift;
    my $retval = ();
    my @output = ();
    while(1) {
        my ( $command, $uuid, $data ) = &recieve_packet($sock);
        if ( $command ) {
            if ( $command eq "STDOUT" ) {
                push(@output, $data);
            } elsif ( $command eq $readto ) {
                $retval = $data;
                last;
            }
        } else {
            last;
        }
    }
    &pprint(3, "Returning &recieve_until with: $retval, @output\n");
    return($retval, @output);
}

1;
