#!/usr/bin/perl

package Perceus::Nodescript;

use Exporter;
#use IO::Socket;
use Getopt::Long;

use vars qw($VERSION @NODES);
$VERSION = "2.0";

use Perceus::Include;
use Perceus::Connect;
use Perceus::Util;
use Perceus::Network;

use Perceus::Socket;
use Perceus::Packet;

our @ISA = ("Exporter");

our @EXPORT = qw(
    $debug
    @NODES
    &create_comm
);

our $debug = 0;

Getopt::Long::Configure ("bundling");

GetOptions(
   'd|debug=s'      => \$debug,
);

push(@NODES, @ARGV);

sub
create_comm1($$)
{
    my $proto               = shift;
    my $class               = ref($proto) || $proto;
    my $hostname            = shift || $NODES[0];
    my $sock                = Perceus::Socket->connect();
    my $packet              = Perceus::Packet->new();
    my $self                = ();

    $packet->command("CONNECT");
    $packet->data($hostname);

    $sock->send($packet->serialize());

    $packet->clear();
    $packet->parse($sock->recieve());

    
    if ( $packet->command() eq "RETVAL" and $packet->data() eq "0" ) {
        $self->{"SOCK"} = $$sock;
        $self->{"XID"} = $packet->xid();
        $self->{"CID"} = $packet->uid();
        &pprint(1, "Created COM object to local perceus daemon with XID: $self->{XID}\n");
    } else {
        &pprint(1, "ERROR: Could not make connect: ". $packet->command() .":". $packet->data() ."\n");
        return();
    }

    bless($self, $class);

    return($self);

}

sub
create_comm($$)
{
    my $proto               = shift;
    my $class               = ref($proto) || $proto;
    my $hostname            = shift || $NODES[0];
    my $self                = ();
    my $pid                 = &gen_uuid();
    my $sock                = &perceusd_local();

    &send($sock, "CONNECT", $hostname, $pid);
    my ($retval, @output) = &recieve_until($sock, "STATUS");

    %{$self} = ();

    if ( $retval eq "OK" ) {
        $self->{"SOCK"} = $$sock;
        $self->{"RID"} = $pid;
        &pprint(1, "Created COM object to local perceus daemon with RID: $pid\n");
    } else {
        print STDERR join("\n", @output)."\n";
        return();
    }

    bless($self, $class);

    return($self);
}


sub
run($$)
{
    my $self = shift;
    my $command = shift;
    my $sock;

    if ( exists($self->{"SOCK"}) ) {
        $sock = \$self->{"SOCK"};
    } else {
        return("255");
    }

    &send($sock, "RUN", $command, $self->{"RID"});

    return(&recieve_until($sock, "RETVAL"));
}

sub
do($$)
{
    my $self = shift;
    my $command = shift;
    my $sock;

    if ( exists($self->{"SOCK"}) ) {
        $sock = \$self->{"SOCK"};
    } else {
        return("255");
    }

    return(&send($sock, "DO", $command, $self->{"RID"}));
}

sub
test($$)
{
    my $self = shift;
    my $command = shift;
    my $sock;

    if ( exists($self->{"SOCK"}) ) {
        $sock = \$self->{"SOCK"};
    } else {
        return("255");
    }

    &send($sock, "TEST", $command, $self->{"RID"});

    my $retval = ( &recieve_until($sock, "RETVAL") )[0];

    # Here we flip the retval; Shell is success with 0, but perl likes it to
    # be defined for true value on conditionals.
    if ( $retval == 0 ) {
        $retval = 1;
    } else {
        $retval = ();
    }

    return($retval);
}

#if ( ! @NODES ) {
#    print STDERR "You must pass the nodename you wish to connect to!\n";
#    exit(1);
#}

1;
