#!/usr/bin/perl

package Perceus::Peer;

use Perceus::Util;
use Exporter;
use strict;

our @ISA = ('Exporter');

our @EXPORT = qw (

);

=head1 NAME

Perceus::Peers - Perceus socket peer interface and datastore

=head1 ABOUT

Perl interface to Perceus peer datastore.

=head1 SYNOPSIS

    use Perceus::Peer;

    my $p = Perceus::Peer->new($key);

    $p->hostname($hostname);
    $p->commit();

    print $p->hostname() ."\n";

    my $obj = get_by_hostname($hostname);
    my @list = get_by_hostname("n00*");

=head1 METHODS

=over 12
=cut








=item new()

Returns a new Perceus::Peer object. It must be called with the unique node
identifier.

=cut
sub
new($$)
{
    my $proto               = shift;
    my $key                 = shift;
    my $class               = ref($proto) || $proto;
    my $self                = ();

    if ( ! $key ) {
        return();
    }

    %{$self} = ();

    $self->{"KEY"} = $key;

    bless($self, $class);

    return $self;
}

=item hostname()

Parses a raw socket string looking for properly formatted packets. Anything
not formated correctly or left over will be returned back.

=cut
sub
hostname($$)
{
    my $self                = shift;
    my $string              = shift;




    return($packet);
}

=item xid()

Will set or print the XID region of the packet. Optional argument will set.
Maximum size is 8 bytes.

=cut
sub
xid($$)
{
    my $self                = shift;
    my $data                = shift;
    my $return              = ();

    if ( $data ) {

        if ( length($data) <= 8 ) {
            $return = 1;
            $self->{"XID"} = $data;
        }

    } else {
        $return = $self->{"XID"};
    }

    return($return);
}

=item uid()

Will set or print the CID region of the packet. Optional argument will set.
Maximum size is 8 bytes.

=cut
sub
uid($$)
{
    my $self                = shift;
    my $data                = shift;
    my $return              = ();

    if ( $data ) {

        if ( length($data) <= 8 ) {
            $return = 1;
            $self->{"CID"} = $data;
        }

    } else {
        $return = $self->{"CID"};
    }

    return($return);
}

=item command()

Will set or print the COMMAND region of the packet. Optional argument will set.
Maximum size is 16 bytes.

=cut
sub
command($$)
{
    my $self                = shift;
    my $data                = shift;
    my $return              = ();

    if ( $data ) {

        if ( length($data) <= 16 ) {
            $return = 1;
            $self->{"COMMAND"} = $data;
        }

    } else {
        $return = $self->{"COMMAND"};
    }

    return($return);
}

=item data()

Will set or print the DATA region of the packet. Optional argument will set.
Maximum size is 1024 bytes.

=cut
sub
data($$)
{
    my $self                = shift;
    my $data                = shift;
    my $return              = ();

    if ( $data ) {

        if ( length($data) <= 1024 ) {
            $return = 1;
            $self->{"DATA"} = $data;
            $self->{"LENGTH"} = length($data);
        }

    } else {
        $return = $self->{"DATA"};
    }

    return($return);
}


=item satisified()

If the packet is properly populated, this will return true.

=cut
sub
satisified($)
{
    my $self                = shift;

    return((exists($self->{"DATA"}) and exists($self->{"LENGTH"}) and $self->{"LENGTH"} == length($self->{"DATA"}) ? 1 : undef) );
}

=item clear()

Clear the object contents.

=cut
sub
clear($)
{
    my $self                = shift;

    %{$self} = ();
}

=item pack()

Pack the object into a string suitable for sending over a socket

=cut
sub
pack($)
{
    my $self                = shift;

#    printf("packed: %6.6s%8.8s%8.8s%16.16s%8.8s%s\n",
#        $MAGIC,
#        $self->{"XID"} || "",
#        $self->{"CID"} || "",
#        $self->{"COMMAND"} || "",
#        $self->{"LENGTH"} || length($self->{"DATA"}),
#        $self->{"DATA"} || ""
#    );
    return(sprintf("%6.6s%8.8s%8.8s%16.16s%8.8s%s",
        $MAGIC,
        $self->{"XID"} || "",
        $self->{"CID"} || "",
        $self->{"COMMAND"} || "",
        $self->{"LENGTH"} || length($self->{"DATA"}),
        $self->{"DATA"} || ""
    ));
}

=back

=head1 SEE ALSO

Perceus::Network, Perceus::Link

=head1 COPYRIGHT

Perceus is copyright Arthur Stevens and Infiscale, Inc.

=cut

1;
