#!/usr/bin/perl
#
# Copyright (c) 2006-2014 Infiscale, Inc. All rights reserved
#

BEGIN {
   require "/etc/perceus/Perceus_Include.pm";
   push(@INC, "$Perceus_Include::libdir");
}

use Perceus::Config;
use File::Basename;
use IO::Socket;
use IO::Interface;

$| = 1;

my $hostname = $ENV{'NODENAME'};
my $output;
my $match = ();
my $global = ();
my %config = &parse_config("/etc/perceus/perceus.conf");


my @configs = qw(
   /etc/sysconfig/network-scripts/ifcfg-
   /etc/sysconfig/nics/./
);

print "hostname $hostname\n";
print "echo $hostname > \$DESTDIR/etc/hostname\n";

my $note;
$note = "";
$note .= "# This configuration has been automatically\n";
$note .= "# generated for $hostname by the\n";
$note .= "# 'ipaddr' Perceus module, and configured in:\n";
$note .= "# /etc/perceus/modules/ipaddr on the head node\n";

# create /etc/network/interfaces for Debian
$start .= "mkdir -p \$DESTDIR" . "/etc/network\n";
$start .= "if [ -f \$DESTDIR/etc/network/interfaces ]; then\n";
$start .= "rm \$DESTDIR/etc/network/interfaces\n";
$start .= "fi\n";
$start .= "echo '$note' >> \$DESTDIR/etc/network/interfaces\n";
$start .= "echo 'auto lo' >> \$DESTDIR/etc/network/interfaces\n";
$start .= "echo 'iface lo inet loopback' >> \$DESTDIR/etc/network/interfaces\n";
$start .= "echo >> \$DESTDIR/etc/network/interfaces\n";

open(IPADDR, "/etc/perceus/modules/ipaddr")
    or die "ERROR: could not open /etc/perceus/modules/ipaddr\n";
while(my $entry = <IPADDR> ) {
    my $string = ();
    chomp $entry;
    $entry =~ s/#.*$//;
    next if ! $entry;

    if ( $entry =~ /^$hostname\s+(.+)$/ ) {
        $string = $1;
        $match = 1;
        $output = ();
    } elsif ( $entry =~ /^([^ ]+)\s+(.+)$/ ) {
        my $regex = $1;
        my $rest = $2;
        $regex =~ s/([^\.]?)\*/$1.*/g;
        if ( $hostname =~ /^$regex$/ ) {
            $string = $rest;
            $global = 1;
        }
    }
    if ( defined($match) or defined($global) ) {
        my @addrs = split(/\s+/, $string);
        foreach my $addr ( @addrs ) {
            my ( %options, %extra_options, $dev, @string, $type);
            my ( @option_keys, @extra_option_keys );
            if ( $addr =~ /^([^\(]+)\(([^\)]+)+\):(.*)/ ) {
                $dev = $1;
                push(@string, $3);
                foreach my $option ( split(/\&/, $2)) {
                    my ($k,$v) = split(/=/, $option, 2);
                    $extra_options{$k} = $v;
                    push(@extra_option_keys, $k);
                }
            } else {
                ($dev, @string) = split(/:/, $addr);
            }
            if ( $dev =~ /^([a-z]+\d)_(\d+)$/ ) {
               $dev = "$1:$2";
               $type = "alias";
            } else {
               $type = "ethernet";
            }
            $options{"DEVICE"} = $dev; push(@option_keys, "DEVICE");
            $options{"ONBOOT"} = "yes"; push(@option_keys, "ONBOOT");
            $options{"TYPE"} = $type; push(@option_keys, "TYPE");
            my ( $ipaddr, $netmask, $gateway ) = split(/\//, join(":", @string));
            if ( $dev and $ipaddr and $netmask ) {
                if ( $ipaddr =~ /^\[(?:default|hostfile)\]$/ ) {
                    $ipaddr = &get_hostentry($hostname, $dev);
                } elsif ( $ipaddr =~ /^\[(?:default|hostfile):(.+)\]$/ ) {
                    my $lookup_host = $1;

                    $lookup_host =~ s/NAME/$hostname/g;
                    $lookup_host =~ s/NIC/$dev/g;
                    $ipaddr = &get_hostentry($lookup_host, $dev);
                } elsif ( $ipaddr =~ /\[nodenum\]/ ) {
                    if ( $hostname =~ /(\d+).*?$/ ) {
                        my $tmpip = sprintf("%d", $1);
                        $ipaddr =~ s/\[nodenum\]/$tmpip/;
                    }
                }
                if ( $netmask =~ /^\[default\]$/ ) {
                    $netmask = &get_netmask();
                } elsif ( $netmask =~ /^\[(?:default:)?(.+)\]$/ ) {
                    my $lookup_host = $1;

                    $lookup_host =~ s/NAME/$hostname/g;
                    $lookup_host =~ s/NIC/$dev/g;
                    $netmask = &get_netmask($lookup_host);
                }
                if ( $gateway =~ /^\[default\]$/ ) {
                    $gateway = &get_ipaddr();
                } elsif ( $gateway =~ /^\[default:(.+)\]$/ ) {
                    my $lookup_host = $1;

                    $lookup_host =~ s/NAME/$hostname/g;
                    $lookup_host =~ s/NIC/$dev/g;
                    $gateway = &get_ipaddr($lookup_host);
                }
            } 
            if ( $dev and $ipaddr eq "dhcp" ) {
                $options{"PROTO"} = "dhcp"; push(@option_keys, "PROTO");
                $options{"BOOTPROTO"} = "dhcp"; push(@option_keys, "BOOTPROTO");
            } elsif ( $dev and ! $ipaddr ) {
                # No default entries
            } else {
                $options{"PROTO"} = "static"; push(@option_keys, "PROTO");
                $options{"BOOTPROTO"} = "static"; push(@option_keys, "BOOTPROTO");
                $options{"IPADDR"} = $ipaddr; push(@option_keys, "IPADDR");
                $options{"NETMASK"} = $netmask; push(@option_keys, "NETMASK");
                $options{"GATEWAY"} = $gateway; push(@option_keys, "GATEWAY");
            }

            $output .= $start if length($output) == 0;
            foreach my $conf ( @configs ) {
                $output .= "mkdir -p \$DESTDIR". dirname($conf) ."\n";
                $output .= "if [ -f \$DESTDIR$conf$dev ]; then\n";
                $output .= "rm \$DESTDIR$conf$dev\n";
                $output .= "fi\n";
                $output .= "echo '$note' >> \$DESTDIR$conf$dev\n";
                foreach my $k ( @option_keys ) {
                    my $option = $options{$k};
                    $output .= "echo '$k=$option' >> \$DESTDIR$conf$dev\n";
                }
                foreach my $k ( @extra_option_keys ) {
                    my $option = $extra_options{$k};
                    $output .= "echo '$k=$option' >> \$DESTDIR$conf$dev\n";
                }
            }
            {
                # and create /etc/network/interfaces for Debian
                if( $options{"ONBOOT"} =~ /yes/i ) {
                    $output .= "echo 'auto $dev' >> \$DESTDIR/etc/network/interfaces\n";
                }
                my $device = delete $options{"DEVICE"};
                my $onboot = delete $options{"ONBOOT"};
                my $type = delete $options{"TYPE"};
                my $proto = delete $options{"PROTO"};
                my $bootproto = delete $options{"BOOTPROTO"};
                my $ipaddr = delete $options{"IPADDR"};
                my $netmask = delete $options{"NETMASK"};
                my $gateway = delete $options{"GATEWAY"};
                $output .= "echo 'iface $dev inet $proto' >> \$DESTDIR/etc/network/interfaces\n";
                if( defined $ipaddr ) {
                    $output .= "echo '  address $ipaddr' >> \$DESTDIR/etc/network/interfaces\n";
                }
                if( defined $netmask ) {
                    $output .= "echo '  netmask $netmask' >> \$DESTDIR/etc/network/interfaces\n";
                }
                if( defined $gateway ) {
                    $output .= "echo '  gateway $gateway' >> \$DESTDIR/etc/network/interfaces\n";
                }
                # and handle any extra options.
                foreach my $k ( @extra_option_keys ) {
                    my $lc_key = lc($k);
                    my $option = $extra_options{$k};
                    $output .= "echo '  $lc_key $option' >> \$DESTDIR/etc/network/interfaces\n";
                }
                $output .= "echo >> \$DESTDIR/etc/network/interfaces\n";
           }
        }
    }
    if ( defined($match) ) {
        last;
    }
}
close IPADDR;

sub get_default_nic {

   return($config{"master network device"}[0]);
}

sub get_netmask {

    my $dev = shift || &get_default_nic;

    my $s                = IO::Socket::INET->new(Proto => 'udp');
    my $return           = $s->if_netmask($dev);

    return $return;

}

sub get_ipaddr {

    my $dev = shift || &get_default_nic;

    my $s                = IO::Socket::INET->new(Proto => 'udp');
    my $return           = $s->if_addr($dev);

    return $return;

}

sub get_hostentry {
    my $name = shift;
    my $device = shift;
    my $ipaddr;
    my $entry;

    open(HOSTS, "/etc/hosts")
        or die "ERROR: could not open /etc/hosts\n";
    while(my $entry = <HOSTS> ) {
        chomp $entry;
        $entry =~ s/#.*$//;
        next if ! $entry;
        if ( $entry =~ /^([^\s]+).*\s+$name(\s|$)/ ) {
            $ipaddr = $1;
            last;
        }
    }
    close HOSTS;
    if ( ! defined $ipaddr ) {
       warn "WARNING: Did not locate IP Address for $name:$device, defaulting to DHCP\n";
       print "echo\n";
       print "echo 'WARNING: There is no entry for \"$name\" in the masters /etc/hosts file thus'\n";
       print "echo 'WARNING: DHCP will be used instead for device '$device'.'\n";
       print "sleep 2\n";
       $ipaddr = "dhcp";
    }
    return($ipaddr);
}

print $output;


